#include "traywidget.h"

TrayWidget* TrayWidget::_MSelf = nullptr;
TrayWidget::TrayWidget(bool intel, BluetoothDbus *dBus, QWidget *parent):
    QWidget(parent),
    _SessionDBus(dBus),
    _config(new Config(this)),
    _Intel(intel)
{
    qDebug() << Q_FUNC_INFO;

//    窗管适配
    if (_Intel) {
        setProperty("useStyleWindowManager", false);
        MotifWmHints hints;
        hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
        hints.functions = MWM_FUNC_ALL;
        hints.decorations = MWM_DECOR_BORDER;
        XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);
        installEventFilter(this);
        KWindowSystem::setState(this->winId(), NET::SkipTaskbar | NET::SkipPager);
    }
    if (!_Intel) {
        trayicon = new TrayIcon();
        connect(this, &TrayWidget::trayIconStatusChanged, trayicon, &TrayIcon::SetTrayIcon);
        connect(trayicon, &TrayIcon::showTayWidget, this, &TrayWidget::showUI);
        connect(trayicon, &TrayIcon::openBluetoothSettings, this, [=]{
            _SessionDBus->openBluetoothSettings();
        });
        connect(_SessionDBus, &BluetoothDbus::showTrayIcon, this, [=](bool show) {
            trayicon->SetAdapterFlag(show);
            if (!show)
                this->setVisible(show);
        });
        setProperty("useSystemStyleBlur", true);
        this->setAttribute(Qt::WA_TranslucentBackground);
        this->setWindowFlags(Qt::Popup | Qt::FramelessWindowHint);
        _SessionDBus->getTrayIconShow();
    }
    connect(QApplication::desktop(), &QDesktopWidget::resized, this, [=] {
        setWidgetPosition();
    });
    connect(QApplication::desktop(), &QDesktopWidget::primaryScreenChanged, this, [=] {
        setWidgetPosition();
    });

    QGSettings *setttt = new QGSettings(QString("org.ukui.SettingsDaemon.plugins.xsettings").toLocal8Bit());
    connect(setttt,&QGSettings::changed, this, [=] (const QString &key) {
        if (key == "scalingFactor") {
            repaint();
        }
    });


    if(QGSettings::isSchemaInstalled("org.ukui.style")) {
        StyleSettings = new QGSettings("org.ukui.style");
        connect(StyleSettings,&QGSettings::changed,this,&TrayWidget::GSettingsChanges);
    }

    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)) {
        transparency_gsettings = new QGSettings(transparency_id);
    }
    if(QGSettings::isSchemaInstalled(transparency_id)) {
        getTransparency();
        connect(transparency_gsettings,&QGSettings::changed,this,[=](QString key) {
            qDebug() << Q_FUNC_INFO << key;
            if (key == QString(TRANSPARENCY_KEY)) {
                getTransparency();
                this->update();
            }
        });
    }
    QPalette palette;
    if(StyleSettings->get("style-name").toString() == "ukui-light" || StyleSettings->get("style-name").toString() == "ukui-default"){
        palette.setColor(QPalette::Background,QColor(255,255,255, 0));
    }else{
        QColor col(QPalette::Base);
        col.setAlpha(tran);
        palette.setColor(QPalette::Background, col);
    }

    this->setPalette(palette);
    this->setFixedSize(420,520);
    this->setAutoFillBackground(true);
    this->setWindowIcon(QIcon::fromTheme("bluetooth"));
    this->setWindowTitle(tr("bluetooth"));
    pairedDevices = _SessionDBus->getPairedDevices();
    pairedList = _SessionDBus->getPairedDevicesList();
    InitMemberVariables();
    connectUISignalAndSlot();
    layoutAddAllPairedDev();
    if (_Intel) {
        QRect availableGeometry = QApplication::screens().at(0)->geometry();
        qDebug() << availableGeometry.x() << availableGeometry.y();
        move((availableGeometry.x() + (availableGeometry.width() - width())/2), (availableGeometry.y() + (availableGeometry.height() - height())/2));
    }
    emit trayIconStatusChanged(_SessionDBus->isPowered());
    _MBtn->setChecked(_SessionDBus->isPowered());
    this->setFocus();
}

TrayWidget *TrayWidget::self( BluetoothDbus *dbus, bool intel)
{
    qDebug() << Q_FUNC_INFO;
    if (!_MSelf) {
        _MSelf = new TrayWidget(intel, dbus, nullptr);
    }
    return _MSelf;
}

TrayWidget::~TrayWidget()
{

}

void TrayWidget::getTransparency() {
    double proportion = transparency_gsettings->get(TRANSPARENCY_KEY).toDouble();
    proportion = ((proportion > 0.85) ? 1 : (proportion + 0.15));
    qDebug() << Q_FUNC_INFO << proportion;
    tran = proportion * 255;
}

void TrayWidget::onClickMBtn(bool value) {
    _MBtn->hide();
    _MLoad->show();
    _MIconTimer->start(100);
    if (value != _SessionDBus->isPowered())
        _SessionDBus->setDefaultAdapterPower(value);
}

void TrayWidget::openBluetoothSlot(bool v)
{
    emit trayIconStatusChanged(v);
    if (!_Intel && !pairedDevices.size())
        return;
    if (_Intel) {
        if (v) {
            _MStackedWidget->setCurrentIndex(0);
        } else {
            _MStackedWidget->setCurrentIndex(1);
        }
        return;
    }
    _MStackedWidget->setVisible(v);
    lineFrame->setVisible(v);
    if (_MSelf)
        _MSelf->setFixedHeight(v ? selfHeight : 102);
    setWidgetPosition();
    update();
}



void TrayWidget::setWidgetPosition()
{
    #define MARGIN 4

    //menu->exec();
    int width = this->width();
    int height= this->height();
    qDebug()<<"menu 's  width  height"<<width<<height;

    QDBusInterface iface("org.ukui.panel",
                         "/panel/position",
                         "org.ukui.panel",
                         QDBusConnection::sessionBus());
    if (!iface.isValid())
        return;
    QDBusReply<QVariantList> reply=iface.call("GetPrimaryScreenGeometry");
    //reply获取的参数共5个，分别是 主屏可用区域的起点x坐标，主屏可用区域的起点y坐标，主屏可用区域的宽度，主屏可用区域高度，任务栏位置

    if (!iface.isValid() || !reply.isValid() || reply.value().size()<5) {
        qCritical() << QDBusConnection::sessionBus().lastError().message();
        this->setGeometry(0,0,width,height);
    }

//    qDebug()<<reply.value().at(4).toInt();
    QVariantList position_list=reply.value();

    switch(reply.value().at(4).toInt()){
    case 1:
        //任务栏位于上方
        kdk::WindowManager::setGeometry(this->windowHandle(),
                                        QRect(position_list.at(0).toInt()+position_list.at(2).toInt()-width-MARGIN,
                                              position_list.at(1).toInt()+MARGIN,
                                              width,
                                              height));
        break;
        //任务栏位于左边
    case 2:
        kdk::WindowManager::setGeometry(this->windowHandle(),
                                        QRect(position_list.at(0).toInt()+MARGIN,
                                              position_list.at(1).toInt()+reply.value().at(3).toInt()-height-MARGIN,
                                              width,
                                              height));
        break;
        //任务栏位于右边
    case 3:
        kdk::WindowManager::setGeometry(this->windowHandle(),
                                        QRect(position_list.at(0).toInt()+position_list.at(2).toInt()-width-MARGIN,
                                              position_list.at(1).toInt()+reply.value().at(3).toInt()-height-MARGIN,
                                              width,
                                              height));
        break;
        //任务栏位于下方
    default:
        kdk::WindowManager::setGeometry(this->windowHandle(),
                                        QRect(position_list.at(0).toInt()+position_list.at(2).toInt()-width-MARGIN,
                                              position_list.at(1).toInt()+reply.value().at(3).toInt()-height-MARGIN,
                                              width,
                                              height));
        break;
    }
}

void TrayWidget::GSettingsChanges(const QString &key)
{
    qDebug() << Q_FUNC_INFO << key;
    if(key == "styleName"){
        QPalette palette;
        QPalette pl;
        QPalette _pl;
        getTransparency();
        if(StyleSettings->get("style-name").toString() == "ukui-light" || StyleSettings->get("style-name").toString() == "ukui-default"){
            _pl.setColor(QPalette::WindowText,Qt::black);
            palette.setColor(QPalette::Background,QColor(255,255,255,0));
            pl.setColor(QPalette::WindowText, QColor(0,0,0,20));
        }else{
            QColor col(QPalette::Base);
            col.setAlpha(tran);
            _pl.setColor(QPalette::WindowText,Qt::white);
            palette.setColor(QPalette::Background, col);
            pl.setColor(QPalette::WindowText, QColor(255,255,255,50));
        }
        this->setPalette(palette);
        _MLabel->setPalette(_pl);
        lineFrame->setPalette(pl);
        lineFrame2->setPalette(pl);
        this->update();
    }
    if (key == "switch") {
        if (_config.gsetting->get("switch").toString() != _SessionDBus->isPowered())
            _MBtn->setChecked(_config.gsetting->get("switch").toBool());
    }
}

void TrayWidget::InitMemberVariables()
{
    _MBtn           = new SwitchButton(this);
    _MLoad          = new QLabel(this);
    _MStackedWidget = new QStackedWidget(this);
    _MLabel         = new QLabel(this);
    _MMainLayout    = new QVBoxLayout(this);
    _MSettingBtn    = new BluetoothSettingLabel(this);
    _MIconTimer = new QTimer(this);

    connect(_MIconTimer,&QTimer::timeout,this,[=]{
        if (iconFlag == 0)
            iconFlag = 7;
        iconFlag--;
        QPixmap pxmp = QIcon::fromTheme("ukui-loading-" + QString::number(iconFlag) + "-symbolic").pixmap(18,18);
        if(StyleSettings->get("style-name").toString() == "ukui-dark")
            pxmp = _config.loadSvg(pxmp, Config::PixmapColor::WHITE);
        _MLoad->setPixmap(pxmp);
        update();
    });

    _MLoad->setVisible(false);

    connect(_MSettingBtn, &BluetoothSettingLabel::clickSignal, this, [=]{
        _SessionDBus->openBluetoothSettings();
    });
    connect(_MBtn, SIGNAL(checkedChanged(bool)), this, SLOT(openBluetoothSlot(bool)));
    connect(_MBtn, SIGNAL(clickedButton(bool)), this, SLOT(onClickMBtn(bool)));

    QHBoxLayout *topLayout = new QHBoxLayout();
    topLayout->setContentsMargins(24,16,25,12);
    topLayout->setSpacing(0);

    InitErrorWidget();
    InitNomalWidget();

    _MStackedWidget->setFixedSize(420,445);
    _MStackedWidget->addWidget(_NomalWidget);
    if (_Intel)
        _MStackedWidget->addWidget(_ErrorWidget);
    _MStackedWidget->setCurrentIndex(0);

    _MLabel->resize(60,32);
    QPalette _pl;
    if(StyleSettings->get("style-name").toString() == "ukui-light" || StyleSettings->get("style-name").toString() == "ukui-default"){
        _pl.setColor(QPalette::WindowText, Qt::black);
    }else{
        _pl.setColor(QPalette::WindowText, Qt::white);
    }
    _MLabel->setPalette(_pl);
    _MLabel->setText(tr("Bluetooth"));

    _MMainLayout->setMargin(0);
    _MMainLayout->setContentsMargins(0,0,0,0);
    _MMainLayout->setSpacing(0);

    topLayout->addWidget(_MLabel);
    topLayout->addStretch(1);
    topLayout->addWidget(_MBtn);
    topLayout->addWidget(_MLoad);

    lineFrame = new QFrame(this);
    lineFrame->setFrameShape(QFrame::HLine);
    lineFrame->setLineWidth(1);
    connect(_MSettingBtn,&BluetoothSettingLabel::hideFeature,this,[=]{
        _MSelf->hide();
    });

    _MMainLayout->addLayout(topLayout);
    lineFrame2 = new QFrame(this);
    lineFrame2->setFrameShape(QFrame::HLine);
    lineFrame2->setLineWidth(1);
    QPalette pl;
    pl.setColor(QPalette::WindowText, QColor(0,0,0,20));
    if(StyleSettings->get("style-name").toString() != "ukui-light" || StyleSettings->get("style-name").toString() == "ukui-default")
        pl.setColor(QPalette::WindowText, QColor(255,255,255,50));
    lineFrame->setPalette(pl);
    lineFrame2->setPalette(pl);
    _MMainLayout->addWidget(lineFrame2);
    _MMainLayout->addWidget(_MStackedWidget);
    _MMainLayout->addWidget(lineFrame);
    _MMainLayout->addWidget(_MSettingBtn);
}

void TrayWidget::InitNomalWidget()
{
    _NomalWidget    = new QWidget(_MStackedWidget);
    _PairedWidget   = new QWidget(_NomalWidget);
    _PairedLayout   = new QVBoxLayout(_PairedWidget);
    _ScrollArea     = new QScrollArea(_NomalWidget);
    QVBoxLayout *_ErrorWidgetLayout = new QVBoxLayout(_NomalWidget);
    QVBoxLayout *_ErrorWidgetChildLayout = new QVBoxLayout(_NomalWidget);
    QLabel *_ErrorWidgetTipLabel = new QLabel(_NomalWidget);

    _ErrorWidgetLayout->setAlignment(0);
    _ErrorWidgetLayout->setContentsMargins(8,12,8,0);
    _ErrorWidgetLayout->setSpacing(5);

    _ErrorWidgetChildLayout->setAlignment(0);
    _ErrorWidgetChildLayout->setContentsMargins(16,0,0,0);

    _ErrorWidgetTipLabel->resize(420,45);
    _ErrorWidgetTipLabel->setText(tr("My Device"));
    QPalette pl;
    pl.setColor(QPalette::WindowText, QColor("#818181"));
    _ErrorWidgetTipLabel->setPalette(pl);

    _NomalWidget->setFixedSize(420,402);
    _PairedWidget->setFixedWidth(420);
    _PairedWidget->setAttribute(Qt::WA_TranslucentBackground);

    //统一滑动模块的样式
    _ScrollArea->setAttribute(Qt::WA_TranslucentBackground);
    _ScrollArea->setProperty("drawScrollBarGroove",false);
    _ScrollArea->verticalScrollBar()->setProperty("drawScrollBarGroove",false);

    _ScrollArea->setFixedWidth(420);
    _ScrollArea->setWidgetResizable(true);
    _ScrollArea->setAlignment(Qt::AlignCenter);
    _ScrollArea->setWidget(_PairedWidget);
    _ScrollArea->setFrameShape(QFrame::NoFrame);
    _ScrollArea->horizontalScrollBar()->hide();
    _ScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    _ScrollArea->verticalScrollBar()->setSingleStep(17);
    _ScrollArea->setPalette(QColor(Qt::transparent));

    _PairedLayout->setAlignment(0);
    _PairedLayout->setContentsMargins(0,0,0,0);
    _PairedLayout->setSpacing(2);

    _ErrorWidgetChildLayout->addWidget(_ErrorWidgetTipLabel);
    _ErrorWidgetLayout->addLayout(_ErrorWidgetChildLayout);
    _ErrorWidgetLayout->addWidget(_ScrollArea);
}

void TrayWidget::InitErrorWidget()
{
    _ErrorWidget       = new QWidget(_MStackedWidget);
    QHBoxLayout* _ErrorWidgeeLayout = new QHBoxLayout(_ErrorWidget);
    QLabel* _ErrorIcon  = new QLabel(_ErrorWidget);

    _ErrorWidget->setFixedSize(446,420);

    _ErrorWidgeeLayout->setAlignment(0);
    _ErrorWidgeeLayout->setContentsMargins(0,0,0,0);
    _ErrorWidgeeLayout->setSpacing(0);

    _ErrorIcon->setPixmap(QPixmap("/usr/share/ukui-bluetooth/no-bluetooth.svg"));

    _ErrorWidgeeLayout->addWidget(_ErrorIcon,1,Qt::AlignCenter);
    if (!_Intel)
        _ErrorIcon->setVisible(false);
}

void TrayWidget::connectUISignalAndSlot()
{
    connect(_SessionDBus,&BluetoothDbus::devRemoveSignal,this,&TrayWidget::removePairedDevUIByAddress);
    connect(_SessionDBus,&BluetoothDbus::adapterAddSignal,this,&TrayWidget::updatePairedDev);
    connect(_SessionDBus,&BluetoothDbus::powerProgressSignal, this, [=](bool inProgress) {
        _MBtn->setVisible(!inProgress);
        _MLoad->setVisible(inProgress);
        if (inProgress && !_MIconTimer->isActive())
            _MIconTimer->start();
        if (!inProgress && _MIconTimer->isActive())
            _MIconTimer->stop();
    });
    connect(_SessionDBus,&BluetoothDbus::powerChangedSignal,this,[=](bool value){
        qDebug() << Q_FUNC_INFO << value <<__LINE__;
        if (!_Intel)
            setWidgetPosition();
        _MBtn->show();
        _MLoad->hide();
        if (_MIconTimer->isActive())
            _MIconTimer->stop();
        if (value != _MBtn->isChecked())
            _MBtn->setChecked(value);
        //_config.gsetting->set("switch",QVariant::fromValue(value));
        emit trayIconStatusChanged(value);
        _SessionDBus->getDefaultAdapterPairedDevices();
        updatePairedDev();
    });
    connect(_SessionDBus,&BluetoothDbus::AdapterChangedSignal,this,[=](QString address){
        Q_UNUSED(address)
        _MBtn->setChecked(_SessionDBus->isPowered());
        updatePairedDev();
    });
}

void TrayWidget::SendMessage(QString name, bool connected)
{
    QString text;
    if (connected) {
        text = QString(tr("The connection with the Bluetooth device “%1” is successful!").arg(name));

    } else {
        text = QString(tr("Bluetooth device “%1” disconnected!").arg(name));
    }
    Config::SendNotifyMessage(text);
}

/************************************************
 * @brief  已配对的设备添加到布局中显示
 * @param  null
 * @return null
*************************************************/
void TrayWidget::layoutAddAllPairedDev()
{
    bool isNull = true;
    int count = 0;
    qDebug() << Q_FUNC_INFO << pairedDevices.size() << __LINE__  << pairedDevices;
    if (!_PairedWidget->findChildren<QDevItem *>().size()) {
        for (auto item : pairedList) {
            QString name = pairedDevices.value(item).at(0);
            _SessionDBus->getDevConnectStatus(item);
            QDevItem *dev = new QDevItem(item, pairedDevices.value(item), _SessionDBus->isConnected(name), _PairedWidget);
            connect(_SessionDBus, &BluetoothDbus::devConnectStatusSignal, dev, [=](QString address, bool connected) {
                qDebug() << " +++ compare " << address << item;
                if (address != item)
                    return;
                dev->connectedChangedSlot(connected);
            });
            connect(dev, &QDevItem::devConnect, this, [=](bool connectdev){
               if(connectdev)
                  _SessionDBus->devConnect(item);
               else
                   _SessionDBus->devDisconnect(item);
            });
            _PairedLayout->addWidget(dev);
            isNull = false;
            count++;
        }
        _PairedLayout->addStretch();
        if (_Intel)
            return;
        isNull = isNull || !_MBtn->isChecked();
        _MStackedWidget->setVisible(!isNull);
        lineFrame->setVisible(!isNull);
        if (_MSelf) {
            selfHeight =160 + count * 54;
            if (count >= 7)
                selfHeight = 515;
            _MSelf->setFixedHeight((isNull ? 102 : selfHeight));
        }
        update();
        setWidgetPosition();
    }
}

void TrayWidget::layoutAddOnePairedDev(QString name, bool paired)
{
    updatePairedDev();
    if (paired)
        SendMessage(name, paired);
}

void TrayWidget::updatePairedDev() {
    pairedDevices = _SessionDBus->getPairedDevices();
    pairedList = _SessionDBus->getPairedDevicesList();
    clearPairedWidget();
    layoutAddAllPairedDev();
}

/************************************************
 * @brief  清空现在显示的所有配对设备UI
 * @param  null
 * @return null
*************************************************/
void TrayWidget::clearPairedWidget()
{
    qDebug() << Q_FUNC_INFO << __LINE__;
    QLayoutItem *child;
    QDevItem *tmp;
    while ((child = _PairedLayout->takeAt(0)) != 0)
    {
        if(child->isEmpty())
            return;
        if(child->widget())
        {
            tmp = qobject_cast<QDevItem*>(child->widget());
            disconnect(tmp, &QDevItem::devConnect, nullptr, nullptr);
            disconnect(_SessionDBus, &BluetoothDbus::devConnectStatusSignal, nullptr, nullptr);
            child->widget()->setParent(NULL);
        }
        delete child;
        delete tmp;
        tmp = nullptr;
        child = nullptr;
    }
}

/************************************************
 * @brief  根据BluezQt::DevicePtr清空现在显示的配对设备UI
 * @param  devPtr 配对设备指针
 * @return null
*************************************************/
void TrayWidget::removePairedDevUIByAddress(QString address)
{
    qDebug() << Q_FUNC_INFO << pairedDevices << __LINE__;
    if (!pairedList.contains(address))
        return;
    pairedList.removeAll(address);
    if (!pairedDevices.contains(address))
        return;
    qDebug() << Q_FUNC_INFO << address << __LINE__;
    QDevItem *item = _PairedWidget->findChild<QDevItem *>(address);
    if (item) {
        if(item->isConnected())
            SendMessage(pairedDevices.value(address).at(0), false);
        _PairedLayout->removeWidget(item);
        item->setParent(NULL);
        delete item;
        item = nullptr;
    }
    pairedDevices.remove(address);
    updatePairedDev();
}

void TrayWidget::showUI()
{
    if (!_Intel) {
        int count = pairedList.size();
        selfHeight =160 + count * 54;
        if (count >= 7)
            selfHeight = 515;
        this->setFixedHeight(((_MBtn->isChecked()) && (count != 0) ?  selfHeight : 102));
        showNormal();
        setWidgetPosition();
        return;
    }
    QApplication::desktop()->update();
    QRect availableGeometry = QApplication::screens().at(0)->geometry();
    qDebug() << availableGeometry.x() << availableGeometry.y();
    move((availableGeometry.x() + (availableGeometry.width() - width())/2), (availableGeometry.y() + (availableGeometry.height() - height())/2));
    showNormal();
}

//用于清楚焦点预选框样式
void TrayWidget::hideWidget() {
    QList<QDevItem*> l = _PairedWidget->findChildren<QDevItem *>();
    if(pressCnt == 1) {
        _MBtn->setFocusStyle(false);
    } else if (pressCnt == l.size() + 2) {
        _MSettingBtn->setFocusStyle(false);
    }
    else {
        l.at(pressCnt - 2)->setFocusStyle(false);
    }
    this->hide();
}

bool TrayWidget::eventFilter(QObject *o, QEvent *e)
{
    if (e->type() == QEvent::ActivationChange)
    {
        if(QApplication::activeWindow() != this)
        {
            hideWidget();
        }
    }
    return QWidget::eventFilter(o,e);
}

void TrayWidget::paintEvent(QPaintEvent *event) {
    Q_UNUSED(event);
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(Qt::transparent);
    QColor col;
    if(StyleSettings->get("style-name").toString() == "ukui-light" || StyleSettings->get("style-name").toString() == "ukui-default"){
        col = QColor(255,255,255);
    } else{
        col = QColor(35,36,38);
    }
    QPainterPath rectPath;

    if (_Intel)
        rectPath.addRect(this->rect());
    else {
        col.setAlpha(tran);
        rectPath.addRoundedRect(this->rect(),12,12);
    }

    painter.setBrush(col);
    painter.drawPath(rectPath);
}

void TrayWidget::keyPressEvent(QKeyEvent *event) {
    return;
    QList<QDevItem*> l = _PairedWidget->findChildren<QDevItem *>();
    int size = l.size();
    switch (event->key()) {
    case Qt::Key_Tab:
    case Qt::Key_Down:
        if (++pressCnt > size + 2)
            pressCnt = 1;
        qDebug() << "press Tab"<< pressCnt;
        if(pressCnt == 1) {
            _MSettingBtn->setFocusStyle(false);
            _MBtn->setFocusStyle(true);
        } else if (pressCnt == size + 2) {
            l.at(size - 1)->setFocusStyle(false);
            _MSettingBtn->setFocusStyle(true);
        }
        else {
             _MBtn->setFocusStyle(false);
            if (pressCnt >= 3)
                l.at(pressCnt - 3)->setFocusStyle(false);
            l.at(pressCnt - 2)->setFocusStyle(true);
        }
        break;
    case Qt::Key_Enter:
    case Qt::Key_Return:
        qDebug() << "press Enter";
        if (_MBtn->hasFocus())
            onClickMBtn(!_MBtn->isChecked());
        break;
    case Qt::Key_Escape:
        qDebug() << "press Esc";
        if(pressCnt == 1) {
            _MBtn->setFocusStyle(false);
        } else if (pressCnt == size + 2) {
            _MSettingBtn->setFocusStyle(false);
        }
        else {
            l.at(pressCnt - 2)->setFocusStyle(false);
        }
        this->hide();
        break;
    default:
        break;
    }
}
